﻿<?php

/*
*	checkversion element
*	version 1.5.0 22/01/2009
*
*	Copyright Ewout Wierda  2008/2009. All rights reserved. 
*	This program is free software: you can redistribute it and/or modify it under 
*	the terms of the GNU General Public License version 3 as published by the 
*	Free Software Foundation, see <http://www.gnu.org/licenses/>.
*	This program is distributed in the hope that it will be useful, but WITHOUT 
*	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*	or FITNESS FOR A PARTICULAR PURPOSE.  See the GPL for more details.
*	Free for non-commercial use, payment for commercial use on request.
*	mail@joomlaloft.com
*	joomlaloft.com
*
*	This version element creates a version checking parameter type for Joomla xml 
*	installation files. It checks the extension's xml instalation file for information about 
*	the extension, and also checks an external file residing on the developer's server 
*	for new versions as well as for updated and extended information. This version 
*	element is inspired by version_check.php by Peter van Westen,  http://www.nonumber.nl. 
*
*	Instructions for use:
*	1. First make it part of the extension in the same way as other elements. 
*	2. Then ensure that there is a folder called 'versioninfo' in the domain stated in the 'authorUrl' tag 
*	of the extension's xml installation file, with a file named exactly the same as the name tag 
*	of the extension's xml installation file, without a file extension. The domain cannot be 
*	changed, but both the folder and the name can be changed if optional attributes are used 
*	as explained below. This file holds new version and extended information, and except for 
*	the name and version all items of information in this file will override the information taken 
*	from the xml installation file. By way of example, it takes the following form in which 
*	everything except the version number is optional and subject to change after the equal sign: 
*	1.5.0|name=Extension*newname=MyExtension*author=MyCompany*newdate=01/01/2000
*	*copyright=(c)2009 MyName all rights reserved.*licence=GPL*commercial=(non-commercial)
*	*email=mail@mywebsite.com*website=http://www.mywebsite.com*demo=http://www.
*	mywebsite.com/demo*documentation=http://www.mywebsite.com/documentation
*	*support=http://www.mywebsite.com/support*download=http://www.mywebsite.
*	com/download/myextension*jed=http://extensions.joomla.org/extensions/owner/
*	myname/1*instructions=Download and install using the Joomla installer.*changelog
*	=Various bugs bugged the previous version and those bugging buggers were removed.
*	3. When a new version becomes available, ensure that it is available as a download a folder  
*	called 'updates' in the domain stated in the 'authorUrl' tag of the extension's xml installation  
*	file. The name of the download file must start with the exactly the same as the name tag of  
*	the extension's xml installation file, followed by 'v' and the new version number, and it must  
*	end with the extension '.zip'. The folder and name can be changed if optional attributes are  
*	used as explained below.
*	4. Finally  insert a tag for this element in the xml installation file. This tag takes the following 
*	form: <param name="modules" type="version" default="[mod or plg]_[fill_in_name_of_extension]" 
*	label="[VERSION_LABEL]" description="[VERSION_HELP_DESCRIPTION]" />. The default value must 
*	start with 'mod_' for a module and 'plg_' for a plugin, and should be the same as the 'Module 
*	Type' shown in the administrator edit extension view. The label and description can be 
*	modified as desired and if the relative strings are added to a language file can also be 
*	translated. Recommended language strings are: 'VERSION_LABEL=Version information'; and  
*	'VERSION_HELP_DESCRIPTION=Information about this extension showing the current version  
*	and other details and checking automatically if a new version is available'. Optionally the folder  
*	where the file with new version and extended information is stored can be changed by adding  
*	an attribute 'upd_path="newfolder"'. The folder and file where the extension can be downloaded  
*	can also be changed optionally by adding 'dl_path="newfolder"' (without starting or trailing  
*	slashes) and 'dl_file="newfile.ext"'.
*	5. Optionally, to enable translation of language strings, the various items of text used in this 
*	element can be added to a language file. Doing so is recommended to avoid that the Joomla 
*	translation engine already finds a match. Taking en-GB as an example, the following language 
*	strings should be added: by=by Version=Version Copyright=Copyright Licence=Licence of=of 
*	Updates=Updates no update info available=no update info available no later version available=
*	no later version available a new version is available!=a new version is available! Changes=
*	Changes Installation=Installation website=website demo=demo documentation=documentation 
*	support=support download=download If you like this extension, please leave a vote or review 
*	at the=If you like this extension, please leave a vote or review at the
*
*	Still planned: converting the external text file to an xml file.
*
*/

defined('JPATH_BASE') or die();

/**
 * Renders a versions and update check element
 */

class JElementVersion extends JElement
{

	var	$_name = 'Version';


	function cleanString( $str = '' )
	{
		$str = preg_replace( '#[^0-9\.]#', '', $str );
		return $str;
	}


	function versionInfo(&$node,$value)
	{

		//set versions in case of failure
		$updates = array();

		//get the extension name from the version check parameter
		//$ext_name = $node->attributes('ext_name');//USE REQUEST AND DBASE QUERY
		//$ext_name = 'mod_onanypagebeta';
		$ext_name = $value;//USE REQUEST AND DBASE QUERY

		//prepare for getting the extension's path
		if (strpos($ext_name,'mod')!==false) {
			//$ext_type = 'mod0_xml';
			$clientId = JApplication::getClientId();
			$ext_type = ($clientId == 1) ? 'mod1_xml' : 'mod0_xml';
		} else {
			$ext_type = 'plg_xml';
		}
		//get the path to the extension
		$xmlpath = JApplicationHelper::getPath($ext_type,$ext_name);
		//open the xml installation file of the extension
		$current_info = JApplicationHelper::parseXMLInstallFile($xmlpath);

		//get the current version, name and url from the version tags for fuirther processing
		$current_version = $current_info['version'];
		$extName = $current_info['name'];
		$authorU = $current_info['authorUrl'];
		$authorUr = ( preg_match('#(((www\.(([a-z]|[A-Z]|[0-9])*\.))+|((([a-z]|[A-Z]|[0-9])*\.)+(([a-z]|[A-Z]|[0-9])*\.)+(([a-z]|[A-Z]|[0-9])*\.)*)){1}[a-zA-Z])#', $authorU) ) ? $authorU : 'www.'.$authorU;
		$authorUrl = ( preg_match('#^((http\://)((www\.(([a-z]|[A-Z]|[0-9])*\.))+|((([a-z]|[A-Z]|[0-9])*\.)+(([a-z]|[A-Z]|[0-9])*\.)+(([a-z]|[A-Z]|[0-9])*\.)*)){1}[a-zA-Z])#', $authorUr) ) ? $authorUr : 'http://'.$authorUr;

		// add extended informaton to the array
		$updates['name'] = $extName;
		$updates['author'] = $current_info['author'];
		$updates['date'] = $current_info['creationdate'];
		$updates['copyright'] = $current_info['copyright'];
		if ($current_info['license']) {
			$updates['licence'] = $current_info['license'];
		}
		$updates['email'] = $current_info['authorEmail'];
		$updates['website'] = $authorUrl;
		// add the path to download updates to the array
		$download_path = ($node->attributes('dl_path')) ? DS.$node->attributes('dl_path') : DS.'versioninfo';
		$download_name = $extName.' v'.$current_version.'.zip';
		$download_file = ($node->attributes('dl_file')) ? DS.$node->attributes('dl_file') : DS.$download_name;
		$download_url = $authorUrl.$download_path.$download_file;
		$updates['download'] = $download_url;

		//to find the latest version info create the url of the new version file from the authorUrl and name tags
		$versions_path = ($node->attributes('upd_path')) ? $node->attributes('upd_path') : 'versioninfo';
		$latest_version_url = $authorUrl.DS.$versions_path;
		$latest_version_file = $latest_version_url.DS.$extName;
		$external_version_file = preg_replace('/\s/u', '%20', $latest_version_file);

		//set to negative to enable failsafe below
		$latest_version = -1;
		//make sure that the new version file is set and the current version is read
		$external_version_headers = @get_headers($external_version_file);
		if (preg_match("|200|", $external_version_headers[0])) {
			//set both version array values to the currently used version
			$updates['current'] = $current_version;
//at some point try to change the external file to xml
//$xml = & JFactory::getXMLParser('Simple');
/*jimport ('joomla.utilities.simplexml');
$xml = new JSimpleXML();
$xml->loadFile($xmlpath);*/
			// open the new version file
			$version_text = file_get_contents( $external_version_file );
			// split if there is additional info
			$extended = strpos($version_text, '|');
			if ($extended === false) {
				// set latest version
				$latest_version = $version_text;
			} else {
				// set latest version and add additional info from the file to the array
				$ext_version_text = explode('|', $version_text, 2);
				$latest_version = $ext_version_text[0];
				$ext_info = (strpos($ext_version_text[1],'*') !== false) ? explode('*', $ext_version_text[1]) : '';
				if ($ext_info !== '') {
					for($i = 0; $i <= count($ext_info); $i++){
						$ext_info_i = ( strpos($ext_info[$i],'=') ) ? explode('=', $ext_info[$i]) : '';
						if ($ext_info_i !== '') {
							$updates[$ext_info_i[0]] = $ext_info_i[1];
						} else {
						}
					}
				}
			}
			//set the second value in the version array to the new available version
			$updates['latest'] = $latest_version;
		}

		// update value 0 if there is no newer version and 1 if newer version is available
		if ($latest_version == -1) {
			$updates['update'] = '-1';
		} else {
			$current_v = $this->cleanString($current_version);
			$latest_v = $this->cleanString($latest_version);
			if ( $latest_v > $current_v ) {
				$updates['update'] = '1';
			} else {
				$updates['update'] = '0';
			}
		}

		//return the current and latest versions plus download url and name
		return $updates;

	}


	function fetchElement($name, $value, &$node, $control_name)
	{

		//get current and latest versions and update value plus download url and name
		$version_info = JElementVersion::versionInfo(&$node,$value);

		//=name
		//=author
		//email
		//name, version, date, copyright and licence of this extension
		$element .= '<span style="font-weight:bold;">'.$version_info['name'].'</span> '.JText::_('by').' <span style="color:#676767; font-weight:bold;"><a href="mailto:'.$version_info['email'].'?subject='.$version_info['name'].'%20version%20'.$version_info['current'].'%20support%20request">'.$version_info['author'].'</a></span>';
		//=current
		//=date
		$element .= '<br/><span style="color:#676767; font-weight:bold;">'.JText::_('Version').' </span>'.$version_info['current'].' '.JText::_('of').' '.$version_info['date'];
		//=copyright
		//licence
		//commercial
		$element .= '<br/><span style="color:#676767; font-weight:bold;">'.JText::_('Copyright').': </span>'.$version_info['copyright'];
		$element .= ($version_info['licence']) ? '<br/><span style="color:#676767; font-weight:bold;">'.JText::_('Licence').': </span>'.$version_info['licence'] : '';
		$element .= ($version_info['commercial'] && $version_info['licence']) ? ' '.$version_info['commercial'] : '';

		//latest
		//set update message 
		$element .= '<br/><br/><span style="color:#676767; font-weight:bold;">'.JText::_('Updates').': </span>';
		//when there is no version info
		if ($version_info['update'] == '-1') {
			$element .= '<span style="color:red;">'.JText::_('no update info available').'</span>';
		//when the current version is the latest version
		} else if ($version_info['update'] == '0') {
			$element .= '<span style="color:green;">'.JText::_('no later version available').'</span>';
		//when there is a later version than the current version
		} else if ($version_info['update'] == '1') {
			$element .= '<span style="color:darkorange; font-weight:bold;">'.JText::_('a new version is available!').'</span>';
			$element .= '<br/><span style="font-weight:bold;">'.'<a href="'.$version_info['download'].'" target="_blank">'.$version_info['newname'].' version '.$version_info['latest'].' '.JText::_('of').' '.$version_info['newdate'].'</a></span>';
		//changelog
		//instructions
			$element .= ($version_info['changelog']) ? '<br/><br/><span style="color:#676767; font-weight:bold;">'.JText::_('Changes').': </span>'.$version_info['changelog'] : '';
			$element .= ($version_info['instructions']) ? '<br/><br/><span style="color:#676767; font-weight:bold;">'.JText::_('Installation').': </span>'.$version_info['instructions'] : '';
		}

		//=website_url
		//demo_url
		//documentation_url
		//support_url
		//=download_url
		// website | demo | documentation | support | download 
		$element .= /*($version_info['changelog'] || $version_info['instructions']) ? '' : */'<br/><br/>';
		$element .= '<a href="'.$version_info['website'].'" target="_blank">'.JText::_('website').'</a>';
		$element .= ($version_info['demo']) ? ' | <a href="'.$version_info['demo'].'" target="_blank">'.JText::_('demo').'</a>' : '';
		$element .= ($version_info['documentation']) ? ' | <a href="'.$version_info['documentation'].'" target="_blank">'.JText::_('documentation').'</a>' : '';
		$element .= ($version_info['support']) ? ' | <a href="'.$version_info['support'].'" target="_blank">'.JText::_('support').'</a>' : '';
		$element .= ' | <a href="'.$version_info['download'].'" target="_blank">'.JText::_('download').'</a>';
		$element .= ' | <a href="'.$version_info['donate'].'" target="_blank">'.JText::_('donate').'</a>';

		$element .= '<br/><br/>';
		if ($version_info['jed']) {
			$element .= '<span style="color:#676767; font-weight:bold;">'.JText::_('If you like this extension, please leave a vote or review at the').' <a href="'.$version_info['jed'].'" target="_blank">Joomla Extensions Directory</a>!</span>';
		$element .= '<br/><br/>';
		}

		return $element;

	}


	function fetchTooltip($label, $description, &$node, $control_name, $name) {

		// label
		$tooltip .= '<label id="'.$control_name.$name.'-lbl" for="'.$control_name.$name.'"';
		if ($description) {
			$tooltip .= ' class="hasTip" title="'.JText::_($label).'::'.JText::_($description).'">';
		} else {
			$tooltip .= '>';
		}
		$tooltip .= JText::_( $label ).'</label>';

		return $tooltip;

	}


}
